#!/usr/bin/env python
# coding: utf-8

# # Python Thetis Data Level 0 > Level 1

# #### Import Packages

import netCDF4
import pandas as pd
import numpy as np
import os
from datetime import datetime


# ### Define functions

def CTD(filepath,outpath,att):
    
    # Make sure NetCDF files are closed
    try: ncfile.close() 
    except: pass
    
    try:
        # Read file into dataframe
        df = pd.read_csv(filepath, sep="	", header=5)

        # Add depth and limit to 50m of data
        df["Depth (m)"] = (df.index * 0.1)+0.1
        df = df.head(500)

        # Get the timestamp and date
        dt = int(df['Timestamp (s)'].mean())
        date = datetime.utcfromtimestamp(dt).strftime('%Y%m%d')
        
        file = outpath+"/"+date+"_"+outpath.split("/")[-1]+".nc"

        # Load NetCDF file or create new
        if os.path.isfile(file):
            ncfile = netCDF4.Dataset(file,mode='a',format='NETCDF4_CLASSIC')
            # Check not duplicate run
            if float(dt) in ncfile.variables['time'][:]:
                print("Duplicate run. Exiting...")
                return "Move"

        else:
            ncfile = createnetcdf(df,file,"temp","degree_C","lake_water_temperature",att)

        # Add data to NetCDF file
        temp = ncfile.variables['temp']
        temp[temp.shape[0],:] = df["Temperature"].to_numpy()

        # Add timestamp to NetCDF file
        time = ncfile.variables['time']
        time[-1] = dt   

        # Close NetCDF file
        ncfile.close() 
        print("Succeded")

        return "Move"
    except Exception as e: 
        print(e)
        return "Stay"

def createnetcdf(df,file,var,unit,name,att):
    
    # Create new file
    ncfile = netCDF4.Dataset(file,mode='w',format='NETCDF4_CLASSIC') 

    # Create dimensions
    depth_dim = ncfile.createDimension('depth', 500)
    depth_time = ncfile.createDimension('time', None)

    # Create variables
    depth = ncfile.createVariable('depth', np.float32, ('depth',))
    depth.units = 'm'
    depth.long_name = 'depth'
    
    time = ncfile.createVariable('time', np.float64, ('time',))
    time.units = 'seconds since 1970-01-01 00:00:00'
    time.long_name = 'time'
    
    v = ncfile.createVariable(var,np.float64,('time','depth')) # note: unlimited dimension is leftmost
    v.units = unit
    v.standard_name = name
    
    # Add depth
    depth[:] = df["Depth (m)"].to_numpy()
    
    # Add global attributes
    ncfile.title = att["title"]
    ncfile.institution = att["institution"]
    ncfile.source = att["source"]
    ncfile.references = att["references"]
    ncfile.history = att["history"]
    ncfile.conventions = att["conventions"]
    ncfile.comment = att["comment"]
        
    return ncfile


# ### Main Script

#Testing (Local Paths)
#inpath = "data"
#prpath = "processed"
#outfolder = "out"

#Testing (Renku Paths)
#inpath = "../../../../data/Level0/00001_ThetisDecimatedData/Incoming"
#prpath = "../../../../data/Level0/00001_ThetisDecimatedData/Processed"
#outfolder = "../../../../data/Level1"

#Production (Server)
inpath = "data/Level0/00001_ThetisDecimatedData/Incoming"
prpath = "data/Level0/00001_ThetisDecimatedData/Processed"
outfolder = "data/Level1"

files = os.listdir(inpath)

for x in range(len(files)):
    move = "Stay" 
    file = files[x]
    ft = file.split("_")[-1].split(".")[0]
    filepath = inpath+"/"+file
    if ft == "CTD":
        # Set attributes
        att = {
            "title":"10001_ThetisTemperature",
            "institution":"EPFL",
            "source":"Add some source",
            "references":"Add some reference",
            "history":"See history on Renku",
            "conventions":"CF 1.7",
            "comment":"Some comment here"
        }
        outpath = outfolder+"/10001_ThetisTemperature"
        move = CTD(filepath,outpath,att)
    if move == "Move":
        # Move file to processed
        os.rename(filepath, prpath+"/"+file)